define(["require", "exports", "./Settings"], function (require, exports, Settings_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Resolve any parent references for the settings, using the `lookup` collection of settings to lookup
     * parents by name.
     *
     * The process starts with the settings specified by `target`. Settings are merged from the oldest
     * parent forward, up to and including the source settings.
     */
    function resolveSettingsParents(lookup, target) {
        var collectedSettings = Settings_1.getParentSettingsChain(lookup, target);
        //  merge the hierarhcy
        return Settings_1.mergeSettings.apply(void 0, collectedSettings);
    }
    var settingsGrandParentName = 'settingsGrandParent';
    var settingsGrandParent = {
        root: {
            fontFamily: 'Helvetica',
            fontWeight: 'light',
            fontSize: 12
        }
    };
    var settingsParentName = 'settingsParent';
    var settingsParent = {
        _parent: settingsGrandParentName,
        root: {
            fontFamily: 'Verdana',
            fontWeight: 'normal'
        }
    };
    var settingsChildName = 'settingsChildName';
    var settingsChild = {
        _parent: settingsParentName,
        root: {
            fontFamily: 'Arial',
            fontSize: 11
        }
    };
    var emptySettingsName = 'emptyLayerName';
    var emptySettings = {};
    var collection = {};
    collection[settingsGrandParentName] = settingsGrandParent;
    collection[settingsParentName] = settingsParent;
    collection[settingsChildName] = settingsChild;
    collection[emptySettingsName] = emptySettings;
    describe('Resolve settings parents tests', function () {
        test('resolveSettingsParents returns undefined when the source settings does not exist', function () {
            var flattened = resolveSettingsParents({}, 'does not exist');
            expect(flattened).toEqual(undefined);
        });
        test('resolveSettingsParents returns undefined when the source settings is empty', function () {
            var flattened = resolveSettingsParents(collection, emptySettingsName);
            expect(flattened).toEqual(undefined);
        });
        test('resolveSettingsParents returns the grand-parent settings when starting with the grand-parent settings', function () {
            var flattened = resolveSettingsParents(collection, settingsGrandParentName);
            expect(flattened).toEqual(settingsGrandParent);
        });
        test('resolveSettingsParents blends the parent and grand-parent layers when starting with the parent settings', function () {
            var flattened = resolveSettingsParents(collection, settingsParentName);
            expect(flattened).toEqual({
                _parent: settingsGrandParentName,
                root: {
                    fontFamily: 'Verdana',
                    fontWeight: 'normal',
                    fontSize: 12
                }
            });
        });
        test('resolveSettingsParents blends the child, parent and grand-parent when starting with child settings', function () {
            var flattened = resolveSettingsParents(collection, settingsChildName);
            expect(flattened).toEqual({
                _parent: settingsParentName,
                root: {
                    fontFamily: 'Arial',
                    fontWeight: 'normal',
                    fontSize: 11
                }
            });
        });
    });
});
//# sourceMappingURL=Settings.parents.test.js.map